/*
 * CavernCamApplet.java
 *
 * Created on July 30, 2007, 10:55 AM
 *
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


import java.applet.*;
import java.awt.*;
import java.net.*;

public class CavernCamApplet extends Applet implements Runnable
{

    private int _imageId = 0;
    private int _dateImageId = 1;
    private int _imageWidth = 800;
    private int _imageHeight = 600;
    private Image _latestImage = null;
    private Image _prevImage = null;
    private Image _latestDateImage = null;
    private Image _prevDateImage = null;
    private int _sleepDuration = 5000;
    private String _imageUrlString = null;
    private String _dateImageUrlString = null;
    private MediaTracker _mediaTracker = null;
    
    private Thread _updateThread = null;
    private boolean _bThreadIdle = true;
    private String _appName = "CavernCamApplet v2.1.1";

    public CavernCamApplet()
    {
    }
    
    public void init()
    {
        _imageUrlString = getParameter("URL");
        if (_imageUrlString == null)
        {
            showStatus("The URL was not specified in the applet parameters. Exiting.");
            return;
        }
        
        _dateImageUrlString = getParameter("dateURL");
        
        String tempString = getParameter("width");
        if (tempString != null)
        {
            try
            {
                int tempInt = Integer.parseInt(tempString);
                this._imageWidth = tempInt;
            }
            catch (Exception ex)
            {
            }
        }
        
        tempString = getParameter("height");
        if (tempString != null)
        {
            try
            {
                int tempInt = Integer.parseInt(tempString);
                this._imageHeight = tempInt;
            }
            catch (Exception ex)
            {
            }
        }
        
        String intervalString = getParameter("Seconds");
        if (intervalString != null)
        {
            try
            {
                int tempInt = Integer.parseInt(intervalString);
                if (tempInt > 0)
                {
                    _sleepDuration = tempInt*1000;
                }
            }
            catch (Exception ex)
            {
            }
        }
    }

    private synchronized boolean isThreadIdle()
    {
        return _bThreadIdle;
    }
    
    private synchronized void setThreadIdle(boolean value)
    {
        _bThreadIdle = value;
    }
    
    public void start()
    {
        setThreadIdle(false);
        _updateThread = new Thread(this);
        _updateThread.start();
    }

    public void stop()
    {
        setThreadIdle(true);
    }

    public void run()
    {
        while (true)
        {
            try
            {
                showStatus(_appName + ": Idle");
                while(isThreadIdle())
                {
                    wait();
                }
            }
            catch (InterruptedException ex)
            {
                showStatus("Interrupted - stopping.");
                break;
            }
            
            try
            {
                // To work around a bug (6439638)that won't send out a GET
                // per browser session where the query string is exactly the
                // same. Solution: add timestamp to string
                
                showStatus(_appName + ": Resolving image URL");
                String fullUrlString = _imageUrlString + "?" + System.currentTimeMillis();
                _latestImage = getImage(new URL(fullUrlString));
            }
            catch (Exception _ex)
            {
                showStatus("Error in obtaining " + _imageUrlString + " - stopping.");
                break;
            }
            
            if (_dateImageUrlString != null)
            {
                try
                {
                    showStatus(_appName + ": Resolving date image URL");
                    String fullUrlString = _dateImageUrlString + "?" + System.currentTimeMillis();
                    _latestDateImage = getImage(new URL(fullUrlString));
                }
                catch (Exception _ex)
                {
                    showStatus("Error in obtaining " + _dateImageUrlString + " - stopping.");
                    break;
                }
            }
 
            try
            {
                showStatus(_appName + ": loading images");
                _mediaTracker = new MediaTracker(this);
                _mediaTracker.addImage(_latestImage, _imageId);
                if (_dateImageUrlString != null)
                    _mediaTracker.addImage(_latestDateImage, _dateImageId);
                _mediaTracker.waitForID(_imageId);
                if (_dateImageUrlString != null)
                    _mediaTracker.waitForID(_dateImageId);
            }
            catch (InterruptedException ex)
            {
                showStatus("Interrupted - stopping.");
                break;
            }
            catch (Exception ex)
            {
                showStatus("Error in displaying image - stopping.");
                break;
            }
            
            showStatus(_appName + ": Painting image");
            repaint();
            System.gc();
            
            try
            {
                showStatus(_appName + ": Sleeping");
                Thread.sleep(_sleepDuration);
                while(isThreadIdle())
                {
                    wait();
                }
            }
            catch (InterruptedException ex)
            {
                showStatus("Interrupted - stopping.");
                break;
            }
            
        }
        showStatus(_appName + ": Finished.");
    }

    public void paint(Graphics g)
    {
        int imageStatus = 0;
        if (_mediaTracker != null)
	{
            imageStatus = _mediaTracker.statusAll(false);
	}
        if (_latestImage != null &&
                ((imageStatus & MediaTracker.COMPLETE) == MediaTracker.COMPLETE))
        {
            _prevImage = _latestImage;
            g.drawImage(_latestImage, 0, 0, this);
            if (_latestDateImage != null)
            {
                _prevDateImage = _latestDateImage;
                g.drawImage(_latestDateImage, this._imageWidth-128, this._imageHeight-32, this);
            }
        }
        else if (_prevImage != null)
        {
            g.drawImage(_prevImage, 0, 0, this);
            if (this._prevDateImage != null)
            {
                g.drawImage(_prevDateImage, this._imageWidth-128, this._imageHeight-32, this);
            }
        }
        else
        {
            g.setColor(Color.BLACK);
            g.fillRect(0,0,_imageWidth,_imageHeight);
        }
    }

}