/*
 * CCUProperties.java
 *
 * Created on July 30, 2007, 3:03 PM
 *
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package caverncamuploader;

import java.util.*;
import java.io.*;

/**
 *
 * @author Robert The Rebuilder
 */
public class CCUProperties
{
    public static final String _versionString = "v2.1.1";
    
    private final String _propFileName = "caverncamuploader.properties";
    private final String _cavernCamDirProperty = "ftp.cavernCamDir";
    private final String _ftpSiteProperty = "ftp.site";
    private final String _imageTypeProperty = "imageType";
    private final String _localImageDirProperty = "localImageDir";
    private final String _matchingSubstringProperty = "matchingSubstring";
    private final String _passwordProperty = "ftp.password";
    private final String _rememberPasswordProperty = "ftp.rememberPassword";
    private final String _usernameProperty = "ftp.username";
    private final String _uploadImageDateProperty = "uploadImageDate";
    private final String _performScreenCaptureProperty = "screenCapture.perform";
    private final String _imageQualityProperty = "screenCapture.imageQuality";
    private final String _uploadIntervalProperty = "uploadInterval";
    private final String _leftXProperty = "screenCapure.leftX";
    private final String _topYProperty = "screenCapture.topY";
    private final String _imageWidthProperty = "image.width";
    private final String _imageHeightProperty = "image.height";
    private final String _webSiteUrlProperty = "website.url";
    private final String _webSiteTitleProperty = "website.title";
    private final String _refreshIntervalProperty = "website.refreshInterval";
    private final String _includeLinksProperty = "website.includeLinks";
    private final String _restartFtpSessionProperty = "ftp.restartSession";
    private final String _awayImageProperty = "awayImage";
    
    
    private Properties _properties = new Properties();
    private static CCUProperties _instance = null;
    private String _fullPropFileName = null;
    private String _fullPath = null;
    
    /** Creates a new instance of CCUProperties */
    private CCUProperties()
    {
    }
    
    public static CCUProperties instance()
    {
        if (_instance == null)
        {
            _instance = new CCUProperties();
        }
        return _instance;
    }
    
    public void read()
    {
        // Form the filename from the current path
        File propFile = new File(_propFileName);
        _fullPropFileName = propFile.getAbsolutePath();
        int lastSeparatorPos = _fullPropFileName.lastIndexOf(File.separator);
        this._fullPath = _fullPropFileName.substring(0,lastSeparatorPos);
        
        // Check if the file exists
        if (propFile.exists())
        {
            // Load the properties
            try
            {
                InputStream fileInputStream = new BufferedInputStream(new FileInputStream(propFile));
                _properties.load(fileInputStream);
                fileInputStream.close();
                fileInputStream = null;
            }
            catch (Exception ex)
            {
                // Stop attempting to get the properties - user must provide values
            }
        }
        
        
        // Add missing properties from file introduced in v2.1
        
        
        // Check whether to remember the password
        String value = this.getRememberPassword();
        if (value == null || value.trim().length() == 0)
        {
            this.setRememberPassword("true");
        }
        
        // Check whether screen capture property exists
        value = this.getPeformScreenCapture();
        if (value == null || value.trim().length() == 0)
        {
            this.setPerformScreenCapture("false");
        }
        
        // Check whether image quality exists
        value = this.getImageQuality();
        if (value == null || value.trim().length() == 0)
        {
            this.setImageQuality("95");
        }
        
        // Check whether upload interval exists
        value = this.getUploadInterval();
        if (value == null || value.trim().length() == 0)
        {
            this.setUploadInterval("5");
        }
        
        // Check for top-left corner
        value = this.getLeftX();
        if (value == null || value.trim().length() == 0)
        {
            this.setLeftX("0");
        }
        value = this.getTopY();
        if (value == null || value.trim().length() == 0)
        {
            this.setTopY("0");
        }
        
        // Check for dimensions
        value = this.getImageWidth();
        if (value == null || value.trim().length() == 0)
        {
            this.setImageWidth("800");
        }
        value = this.getImageHeight();
        if (value == null || value.trim().length() == 0)
        {
            this.setImageHeight("600");
        }

        // Check for web site properties
        value = this.getWebSiteUrlProperty();
        if (value == null || value.trim().length() == 0)
        {
            this.setWebSiteUrlProperty("http://www.mysite.com/CavernCam");
        }
        value = this.getWebSiteTitleProperty();
        if (value == null || value.trim().length() == 0)
        {
            this.setWebSiteTitleProperty("My CavernCam");
        }
        value = this.getRefreshIntervalProperty();
        if (value == null || value.trim().length() == 0)
        {
            this.setRefreshIntervalProperty("5");
        }
        value = this.getIncludeLinksProperty();
        if (value == null || value.trim().length() == 0)
        {
            this.setIncludeLinksProperty("true");
        }
        
        // Check for the restart session property
        value = this.getRestartFtpSessionProperty();
        if (value == null || value.trim().length() == 0)
        {
            this.setRestartFtpSessionProperty("false");
        }
        
    }
    
    public void write()
    {
        boolean bRememberPassword = this.getRememberPassword().equalsIgnoreCase("true");
        try
        {   
            String password = _properties.getProperty(this._passwordProperty);
            if (!bRememberPassword)
            {
                // Temporarily remove the password property before it is saved
                _properties.remove(this._passwordProperty);
            }
            File propFile = new File(this._fullPropFileName);
            OutputStream fileOutputStream = new BufferedOutputStream(new FileOutputStream(propFile));
            _properties.store(fileOutputStream,"CavernCam Uploader Properties "+this._versionString);
            fileOutputStream.close();
            fileOutputStream = null;
            if (!bRememberPassword)
            {
                // Restore the password property 
                _properties.setProperty(this._passwordProperty,password);
            }
        }
        catch (Exception ex)
        {
            javax.swing.JOptionPane.showMessageDialog(null,"Your settings could not be saved to the file "+this._fullPropFileName+"\nPlease make sure you have write permissions to this folder.",
                    "Save Error",
                    javax.swing.JOptionPane.ERROR_MESSAGE);
        }
    }
    
    public boolean isComplete()
    {
        return (getLocalImageDir() != null &&
                getImageType() != null &&
                getMatchingSubstring() != null &&
                getFtpSite() != null &&
                getCavernCamDir() != null &&
                getUsername() != null);
    }
    
    public String getFullPath()
    {
        return _fullPath;
    }
    
    public String getCavernCamDir()
    {
        return _properties.getProperty(this._cavernCamDirProperty);
    }
    
    public String getFtpSite()
    {
        return _properties.getProperty(_ftpSiteProperty);
    }
    
    public String getImageType()
    {
        return _properties.getProperty(_imageTypeProperty);
    }
    
    public String getLocalImageDir()
    {
        return _properties.getProperty(_localImageDirProperty);
    }
    
    public String getMatchingSubstring()
    {
        return _properties.getProperty(_matchingSubstringProperty);
    }
    
    public String getPassword()
    {
        return _properties.getProperty(_passwordProperty);
    }
    
    public String getRememberPassword()
    {
        return _properties.getProperty(this._rememberPasswordProperty);
    }
    
    public String getUsername()
    {
        return _properties.getProperty(_usernameProperty );
    }
    
    public String getUploadImageDate()
    {
        return _properties.getProperty(_uploadImageDateProperty );
    }
    
    public String getPeformScreenCapture()
    {
        return _properties.getProperty(this._performScreenCaptureProperty);
    }
    
    public String getImageQuality()
    {
        return _properties.getProperty(this._imageQualityProperty);
    }
    
    public String getUploadInterval()
    {
        return _properties.getProperty(this._uploadIntervalProperty);
    }
    
    public String getLeftX()
    {
        return _properties.getProperty(this._leftXProperty);
    }
    
    public String getTopY()
    {
        return _properties.getProperty(this._topYProperty);
    }
    
    public String getImageWidth()
    {
        return _properties.getProperty(this._imageWidthProperty);
    }
    
    public String getImageHeight()
    {
        return _properties.getProperty(this._imageHeightProperty);
    }
    
    public String getWebSiteUrlProperty()
    {
        return _properties.getProperty(this._webSiteUrlProperty);
    }
    
    public String getWebSiteTitleProperty()
    {
        return _properties.getProperty(this._webSiteTitleProperty);
    }
    
    public String getRefreshIntervalProperty()
    {
        return _properties.getProperty(this._refreshIntervalProperty);
    }
    
    public String getIncludeLinksProperty()
    {
        return _properties.getProperty(this._includeLinksProperty);
    }
    
    public String getRestartFtpSessionProperty()
    {
        return _properties.getProperty(this._restartFtpSessionProperty);
    }
    
    public String getAwayImageProperty()
    {
        return _properties.getProperty(this._awayImageProperty);
    }
    
    public void setCavernCamDir(String value)
    {
        _properties.setProperty(this._cavernCamDirProperty,value);
    }
    
    public void setFtpSite(String value)
    {
        _properties.setProperty(_ftpSiteProperty,value);
    }
    
    public void setImageType(String value)
    {
        _properties.setProperty(_imageTypeProperty,value);
    }
    
    public void setLocalImageDir(String value)
    {
        _properties.setProperty(_localImageDirProperty,value);
    }
    
    public void setMatchingSubstring(String value)
    {
        _properties.setProperty(_matchingSubstringProperty,value);
    }
    
    public void setPassword(String value)
    {
        _properties.setProperty(_passwordProperty,value);
    }
    
    public void setRememberPassword(String value)
    {
        _properties.setProperty(this._rememberPasswordProperty,value);
    }
    
    public void setUsername(String value)
    {
        _properties.setProperty(_usernameProperty ,value);
    }
    
    public void setUploadImageDate(String value)
    {
        _properties.setProperty(_uploadImageDateProperty ,value);
    }
    
    public void setPerformScreenCapture(String value)
    {
        _properties.setProperty(this._performScreenCaptureProperty,value);
    }
    
    public void setImageQuality(String value)
    {
        _properties.setProperty(this._imageQualityProperty,value);
    }
    
    public void setUploadInterval(String value)
    {
        _properties.setProperty(this._uploadIntervalProperty,value);
    }
    
    public void setLeftX(String value)
    {
        _properties.setProperty(this._leftXProperty,value);
    }
    
    public void setTopY(String value)
    {
        _properties.setProperty(this._topYProperty,value);
    }
    
    public void setImageWidth(String value)
    {
        _properties.setProperty(this._imageWidthProperty,value);
    }
    
    public void setImageHeight(String value)
    {
        _properties.setProperty(this._imageHeightProperty,value);
    }
    
    public void setWebSiteUrlProperty(String value)
    {
        _properties.setProperty(this._webSiteUrlProperty,value);
    }
    
    public void setWebSiteTitleProperty(String value)
    {
        _properties.setProperty(this._webSiteTitleProperty,value);
    }
    
    public void setRefreshIntervalProperty(String value)
    {
        _properties.setProperty(this._refreshIntervalProperty,value);
    }
    
    public void setIncludeLinksProperty(String value)
    {
        _properties.setProperty(this._includeLinksProperty,value);
    }
    
    public void setRestartFtpSessionProperty(String value)
    {
        _properties.setProperty(this._restartFtpSessionProperty,value);
    }
    
    public void setAwayImageProperty(String value)
    {
        _properties.setProperty(this._awayImageProperty,value);
    }
}
