/*
 * FtpUploadThread.java
 *
 * Created on August 7, 2007, 9:32 AM
 *
 *
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package caverncamuploader;

import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.image.*;

import com.sun.image.codec.jpeg.*; 

/**
 *
 * @author Robert The Rebuilder
 */
public class FtpUploadThread extends Thread
{
    private final String _uploadFileName = "image.jpg";
    private final String _uploadDateName = "date.txt";
    
    private boolean _shouldIStop = false;
    private String _ftpSite = CCUProperties.instance().getFtpSite();
    private String _username = CCUProperties.instance().getUsername();
    private String _password = CCUProperties.instance().getPassword();
    private String _ftpDir = CCUProperties.instance().getCavernCamDir();
    private String _matchSubstring = CCUProperties.instance().getMatchingSubstring();
    private String _imageType = CCUProperties.instance().getImageType();
    private String _localImageDir = CCUProperties.instance().getLocalImageDir();
    private boolean _bUploadDate = CCUProperties.instance().getUploadImageDate().equalsIgnoreCase("true");
    private boolean _bScreenCapture = CCUProperties.instance().getPeformScreenCapture().equalsIgnoreCase("true");
    private boolean _bRestartFtp = CCUProperties.instance().getRestartFtpSessionProperty().equalsIgnoreCase("true");
    private float _imageQuality = .95f;
    private String _captureImageName = _localImageDir + File.separatorChar + "image.jpg";
    private String _dateImageName = _localImageDir + File.separatorChar + "dateImage.jpg";
    private int _uploadInterval = 5;
    private int _topY = 0;
    private int _leftX = 0;
    private int _imageWidth = 800;
    private int _imageHeight = 600;
    private Main _mainWindow = null;
    private Linlyn _ftp = null;
    
    public FtpUploadThread(Main mainWindow)
    {
        _mainWindow = mainWindow;
    }
    
    public boolean connectFtp()
    {
        if (_ftp != null)
        {
            disconnectFtp();
        }
        // Attempt to connect
        try
        {
            _ftp = new Linlyn(_ftpSite,_username,_password);
        }
        catch (Exception ex)
        {
            if (Linlyn.DEBUG)
            {
                System.out.println(ex.toString());
            }
            java.awt.EventQueue.invokeLater(new ShowErrorRunnable("FTP Error",
                    "Could not log in to the FTP site.  Check that the following is correct:\n\t- FTP Site\n\t- Username\n\t- Password\n" +
                            "Also check whether your connection is still active."));
            return false;
        }
        
        
        // Go to the correct directory
        try
        {
            _ftp.ftpSetDir(_ftpDir);
        }
        catch (Exception ex)
        {
            if (Linlyn.DEBUG)
            {
                System.out.println(ex.toString());
            }
            java.awt.EventQueue.invokeLater(new ShowErrorRunnable("FTP Error",
                    "Could not change to the directory \"" + _ftpDir + " on the FTP site."));
            return false;
        }
        return true;
    }
    
    public void disconnectFtp()
    {
        if (_ftp != null)
        {
            // Attempt to clean up
            try
            {
                _ftp.Logout();
            }
            catch (Exception ex)
            {
                // Not sure what the value is in announcing an error upon logout
            }
            _ftp = null;
        }
    }

    public void run()
    {
        if (!this.connectFtp())
            return;
        
        // Get the upload interval
        String value = CCUProperties.instance().getUploadInterval();
        try
        {
            int tempInt = Integer.parseInt(value);
            this._uploadInterval = tempInt;
        }
        catch (Exception ex)
        {
        }
        
        // Get the screen dimensions
        value = CCUProperties.instance().getImageWidth();
        try
        {
            int tempInt = Integer.parseInt(value);
            this._imageWidth = tempInt;
        }
        catch (Exception ex)
        {
        }
        value = CCUProperties.instance().getImageHeight();
        try
        {
            int tempInt = Integer.parseInt(value);
            this._imageHeight = tempInt;
        }
        catch (Exception ex)
        {
        }
        
        if (this._bScreenCapture)
        {
            // Get the image quality property
            value = CCUProperties.instance().getImageQuality();
            try
            {
                int tempInt = Integer.parseInt(value);
                this._imageQuality = (float)tempInt/100;
            }
            catch (Exception ex)
            {
            }
            
            // Get the top-left corner
            value = CCUProperties.instance().getLeftX();
            try
            {
                int tempInt = Integer.parseInt(value);
                this._leftX = tempInt;
            }
            catch (Exception ex)
            {
            }
            value = CCUProperties.instance().getTopY();
            try
            {
                int tempInt = Integer.parseInt(value);
                this._topY = tempInt;
            }
            catch (Exception ex)
            {
            }
        }
        
        BufferedImage dateImage = null;
        Graphics dateGraphics = null;
        if (this._bUploadDate)
        {
            dateImage = new BufferedImage(128,32,BufferedImage.TYPE_INT_RGB);
            dateGraphics = dateImage.getGraphics();
        }
        
        // Start the monitor loop
        File imageDirFile = null;
        File latestImageFile = null;
        long latestFileModTime = 0;
        DataInputStream dataInputStream = null;
        byte [] bytes = null;
        FileFilter imageFileFilter = new ImageFileFilter();
        File candidateFile = null;
        TreeMap<Long,File> sortedFiles = null;
        ByteArrayOutputStream imageOutputStream = null;
        while (!shouldIStop())
        {
            try
            {
                this.sleep(_uploadInterval*1000);
            }
            catch (Exception ex)
            {
                // Interruption - we should stop
                break;
            }
            
            if (_bScreenCapture)
            {
                imageOutputStream = new ByteArrayOutputStream();
                ScreenCapture sc = new ScreenCapture();
                sc.capture((float)this._imageQuality,imageOutputStream,this._leftX,this._topY,this._imageWidth,this._imageHeight);
                sc = null;
                bytes = imageOutputStream.toByteArray();
                imageOutputStream = null;
            }
            else
            {
                // Get a list of matching image files in the image directory
                imageDirFile = new File(_localImageDir);
                File [] imageFiles = imageDirFile.listFiles(imageFileFilter);
                imageDirFile = null;
                
                // Find one that is more recent than the one we know about
                long candidateLatestFileModTime = latestFileModTime;
                sortedFiles = new TreeMap<Long,File>();
                for (File anImageFile : imageFiles)
                {
                    boolean canBeRead = false;
                    try
                    {
                        canBeRead = anImageFile.canRead();
                    }
                    catch (SecurityException ex)
                    {
                        // The file is busy - keep looking
                    }
                    if (canBeRead)
                    {
                        sortedFiles.put(anImageFile.lastModified(),anImageFile);
                    }
                }
                imageFiles = null;
                
                candidateFile = null;
                if (sortedFiles.size() > 1)
                {
                    // Get the next to last one
                    File tempFile = null;
                    for (File dCandidateFile : sortedFiles.values())
                    {
                        candidateFile = tempFile;
                        tempFile = dCandidateFile;
                    }
                }
                
                if (candidateFile == null)
                {
                    // No new images found - just continue
                    sortedFiles = null;
                    System.gc();
                    continue;
                }
                
                // Load the file into memory
                try
                {
                    dataInputStream = new DataInputStream(new FileInputStream(candidateFile));
                    bytes = new byte[dataInputStream.available()];
                    dataInputStream.read(bytes);
                    dataInputStream.close();
                    dataInputStream = null;
                }
                catch (Exception ex)
                {
                    java.awt.EventQueue.invokeLater(new ShowErrorRunnable("File Error","Could not load the image file."+ex.getMessage()));
                    break;
                }
                
                // Delete all but the newest file
                File tempFile = null;
                candidateFile = null;
                for (File anImageFile : sortedFiles.values())
                {
                    tempFile = candidateFile;
                    candidateFile = anImageFile;
                    try
                    {
                        if (tempFile != null)
                        {
                            tempFile.delete();
                        }
                    }
                    catch (SecurityException ex)
                    {
                        // No dice - this file is busy
                    }
                }
                sortedFiles = null;
            }
            
            // Attempt to upload the image
            if (this._bRestartFtp)
            {
                if (!this.connectFtp())
                    break;
            }
            try
            {
                _ftp.upload(null,this._uploadFileName,bytes,false);
                bytes = null;
            }
            catch (Exception ex)
            {
                java.awt.EventQueue.invokeLater(new ShowErrorRunnable("FTP Error",
                        "Could not upload image to the CavernCam Directory on the FTP site."));
                break;
            }
            
            if (this._bUploadDate)
            {   
                // Get the current GMT date/time as a string
                DateFormat dateFormat = new SimpleDateFormat();
                dateFormat.setTimeZone(new SimpleTimeZone(0,"GMT"));
                String dateString = dateFormat.format(new Date()) + " GMT";
                
                // Render the date into the hidden graphics area
                dateGraphics.setColor(Color.BLACK);
                dateGraphics.fillRect(0,0,128,32);
                dateGraphics.setColor(Color.WHITE);
                dateGraphics.drawString(dateString,10,16);
                
                imageOutputStream = new ByteArrayOutputStream();
                try
                {
                    // Encode the image as a JPEG
                    JPEGImageEncoder snapEncoder = JPEGCodec.createJPEGEncoder(imageOutputStream);
                    JPEGEncodeParam param = snapEncoder.getDefaultJPEGEncodeParam(dateImage);
                    param.setQuality(.9f, false);
                    snapEncoder.setJPEGEncodeParam(param);
                    snapEncoder.encode(dateImage);
                }
                catch (Exception ex)
                {
                    java.awt.EventQueue.invokeLater(new ShowErrorRunnable("JPEG Error",
                            "Could not create date image."));
                    break;
                }
                
                // Attempt to upload the current date iamge
                if (this._bRestartFtp)
                {
                    if (!this.connectFtp())
                        break;
                }
                try
                {
                    _ftp.upload(null,"date_image.jpg",imageOutputStream.toByteArray(),false);
                }
                catch (Exception ex)
                {
                    java.awt.EventQueue.invokeLater(new ShowErrorRunnable("FTP Error",
                            "Could not upload date image to the CavernCam Directory on the FTP site."));
                    break;
                }
                imageOutputStream = null;
                
                // Attempt to upload the current date as a text file
                if (this._bRestartFtp)
                {
                    if (!this.connectFtp())
                        break;
                }
                try
                {
                    _ftp.upload(null,this._uploadDateName,dateString.getBytes(),true);
                }
                catch (Exception ex)
                {
                    java.awt.EventQueue.invokeLater(new ShowErrorRunnable("FTP Error",
                            "Could not upload date to the CavernCam Directory on the FTP site."));
                    break;
                }
            }
            
            
            // Do some much-needed garbage collection
            System.gc();
        }
        
        this.disconnectFtp();
        
        // Notify the main thread
        java.awt.EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                _mainWindow.onFtpThreadFinished();
            }
        });
    }
    
    public synchronized void requestStop()
    {
        _shouldIStop = true;
    }
    
    public synchronized boolean shouldIStop()
    {
        return _shouldIStop;
    }
    
    private class ImageFileFilter implements FileFilter
    {
        public boolean accept(File whichFile)
        {
            String name = whichFile.getName().trim().toLowerCase();
            return (name.startsWith(_matchSubstring) && name.endsWith(_imageType));
        }
    }
    
    private class ShowErrorRunnable implements Runnable
    {
        String _errorType = null;
        String _errorMessage = null;
        public ShowErrorRunnable(String errorType, String errorMessage)
        {
            _errorType = errorType;
            _errorMessage = errorMessage;
        }
        
        public void run()
        {
            _mainWindow.showError(_errorType,_errorMessage);
        }
    }
}

