/*
 * PropertiesForm.java
 *
 * Created on July 27, 2007, 10:55 AM
 *
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package caverncamuploader;

import java.io.*;
import java.util.*;
import java.text.*;

/**
 *
 * @author  Robert The Rebuilder
 */
public class PropertiesForm extends javax.swing.JFrame
{
    private final String _jpgExtension = "jpg";
    
    
    
    /** Creates new form Main */
    public PropertiesForm()
    {
        initComponents();
        refreshFields();
    }
    
    private void refreshFields()
    {
        this._imageTypeComboBox.removeAllItems();
        this._imageTypeComboBox.addItem(_jpgExtension);
        this._imageTypeComboBox.setSelectedIndex(0);
        
        // Set fields based on properties
        String value = null;
        value = CCUProperties.instance().getCavernCamDir();
        if (value != null)
        {
            this._cavernCamDirTextField.setText(value);
        }
        value = CCUProperties.instance().getFtpSite();
        if (value != null)
        {
            this._ftpSiteTextField.setText(value);
        }
        value = CCUProperties.instance().getImageType();
        if (value != null)
        {
            this._imageTypeComboBox.setSelectedItem(value);
        }
        value = CCUProperties.instance().getLocalImageDir();
        if (value != null)
        {
            this._localImageDirTextField.setText(value);
        }
        value = CCUProperties.instance().getMatchingSubstring();
        if (value != null)
        {
            this._matchingSubstringTextField.setText(value);
        }
        value = CCUProperties.instance().getPassword();
        if (value != null)
        {
            this._passwordTextField.setText(value);
        }
        value = CCUProperties.instance().getRememberPassword();
        if (value != null)
        {
            this._rememberPasswordCheckBox.setSelected(value.equalsIgnoreCase("true"));
        }
        value = CCUProperties.instance().getUsername();
        if (value != null)
        {
            this._usernameTextField.setText(value);
        }
        value = CCUProperties.instance().getUploadImageDate();
        if (value != null)
        {
            this._uploadImageDateCheckBox.setSelected(value.equalsIgnoreCase("true"));
        }
        value = CCUProperties.instance().getPeformScreenCapture();
        if (value != null)
        {
            this._performScreenCaptureCheckBox.setSelected(value.equalsIgnoreCase("true"));
            this._imageQualitySlider.setEnabled(this._performScreenCaptureCheckBox.isSelected());
        }
        value = CCUProperties.instance().getImageQuality();
        if (value != null)
        {
            int intValue = 95;
            try
            {
                int tempInt = Integer.parseInt(value);
                intValue = tempInt;
                this._imageQualityTextField.setText(value);
            }
            catch (Exception ex)
            {
            }
            this._imageQualitySlider.setValue(intValue);
            this._imageQualityTextField.setText(String.valueOf(intValue));
        }
        value = CCUProperties.instance().getUploadInterval();
        if (value != null)
        {
            int intValue = 5;
            try
            {
                int tempInt = Integer.parseInt(value);
                intValue = tempInt;
                this._uploadIntervalTextField.setText(value);
            }
            catch (Exception ex)
            {
            }
            this._uploadIntervalSlider.setValue(intValue);
            this._uploadIntervalTextField.setText(String.valueOf(intValue));
        }
        value = CCUProperties.instance().getLeftX();
        if (value != null)
        {
            this._leftXTextField.setText(value);
        }
        value = CCUProperties.instance().getTopY();
        if (value != null)
        {
            this._topYTextField.setText(value);
        }
        value = CCUProperties.instance().getImageWidth();
        if (value != null)
        {
            this._imageWidthTextField.setText(value);
        }
        value = CCUProperties.instance().getImageHeight();
        if (value != null)
        {
            this._imageHeightTextField.setText(value);
        }
        value = CCUProperties.instance().getRestartFtpSessionProperty();
        if (value != null)
        {
            this._restartFtpSessionCheckBox.setSelected(value.equalsIgnoreCase("true"));
        }
        value = CCUProperties.instance().getAwayImageProperty();
        if (value != null)
        {
            this._awayImageTextField.setText(value);
        }
    }
    
    private void readFromFields() throws Exception
    {
        String value = null;
        
        value = this._localImageDirTextField.getText().trim();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the Local Image Directory.");
        }
        File dirTest = new File(value);
        if (!dirTest.isDirectory())
        {
            throw new Exception(value + " is not a valid directory.  Please specify a new Local Image Directory.");
        }
        CCUProperties.instance().setLocalImageDir(value);
        
        value = (String)this._imageTypeComboBox.getSelectedItem();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the Image Type.");
        }
        CCUProperties.instance().setImageType(value.toLowerCase());
        
        value = this._matchingSubstringTextField.getText().trim().toLowerCase();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the Image Name Matching Substring.");
        }
        CCUProperties.instance().setMatchingSubstring(value);
        
        value = this._ftpSiteTextField.getText().trim();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the FTP Site.");
        }
        CCUProperties.instance().setFtpSite(value);
                
        value = this._cavernCamDirTextField.getText().trim();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the CavernCam Directory.");
        }
        CCUProperties.instance().setCavernCamDir(value);
        
        value = this._usernameTextField.getText().trim();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the FTP Username.");
        }
        CCUProperties.instance().setUsername(value);
        
        value = this._passwordTextField.getText().trim();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the FTP Password.");
        }
        CCUProperties.instance().setPassword(value);
        
        value = (this._rememberPasswordCheckBox.isSelected()) ? "true":"false";
        CCUProperties.instance().setRememberPassword(value);
        
        value = (this._uploadImageDateCheckBox.isSelected()) ? "true":"false";
        CCUProperties.instance().setUploadImageDate(value);
        
        value = (this._performScreenCaptureCheckBox.isSelected()) ? "true":"false";
        CCUProperties.instance().setPerformScreenCapture(value);
        
        value = (this._restartFtpSessionCheckBox.isSelected()) ? "true":"false";
        CCUProperties.instance().setRestartFtpSessionProperty(value);
        
        value = this._imageQualityTextField.getText();
        CCUProperties.instance().setImageQuality(value);
        
        value = this._uploadIntervalTextField.getText();
        CCUProperties.instance().setUploadInterval(value);
        
        value = this._imageWidthTextField.getText().trim();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the Image Width.");
        }
        CCUProperties.instance().setImageWidth(value);
        
        value = this._imageHeightTextField.getText().trim();
        if (value.length() == 0)
        {
            throw new Exception("Please specify the Image Height.");
        }
        CCUProperties.instance().setImageHeight(value);
        
        if (this._performScreenCaptureCheckBox.isSelected())
        {
            value = this._leftXTextField.getText().trim();
            if (value.length() == 0)
            {
                throw new Exception("Please specify the Left X Screen Capture coordinate.");
            }
            CCUProperties.instance().setLeftX(value);
            
            value = this._topYTextField.getText().trim();
            if (value.length() == 0)
            {
                throw new Exception("Please specify the Top Y Screen Capture coordinate.");
            }
            CCUProperties.instance().setTopY(value);
        }
        
        value = this._awayImageTextField.getText().trim();
        if (value.length() != 0)
        {
            CCUProperties.instance().setAwayImageProperty(value);
        }
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        _ftpSiteTextField = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        _cavernCamDirTextField = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        _usernameTextField = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        _passwordTextField = new javax.swing.JTextField();
        _rememberPasswordCheckBox = new javax.swing.JCheckBox();
        _uploadImageDateCheckBox = new javax.swing.JCheckBox();
        jLabel9 = new javax.swing.JLabel();
        _uploadIntervalTextField = new javax.swing.JTextField();
        _uploadIntervalSlider = new javax.swing.JSlider();
        _restartFtpSessionCheckBox = new javax.swing.JCheckBox();
        jPanel2 = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        _localImageDirTextField = new javax.swing.JTextField();
        _localImageDirBrowseButton = new javax.swing.JButton();
        _imageTypeComboBox = new javax.swing.JComboBox();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        _matchingSubstringTextField = new javax.swing.JTextField();
        jLabel12 = new javax.swing.JLabel();
        _imageWidthTextField = new javax.swing.JTextField();
        jLabel13 = new javax.swing.JLabel();
        _imageHeightTextField = new javax.swing.JTextField();
        jLabel14 = new javax.swing.JLabel();
        _awayImageTextField = new javax.swing.JTextField();
        _awayImageBrowseButton = new javax.swing.JButton();
        _okButton = new javax.swing.JButton();
        _cancelButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        _performScreenCaptureCheckBox = new javax.swing.JCheckBox();
        jLabel8 = new javax.swing.JLabel();
        _imageQualityTextField = new javax.swing.JTextField();
        _imageQualitySlider = new javax.swing.JSlider();
        _topYTextField = new javax.swing.JTextField();
        jLabel10 = new javax.swing.JLabel();
        _leftXTextField = new javax.swing.JTextField();
        jLabel11 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Settings");
        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("FTP Properties"));
        jLabel1.setText("FTP Site:");

        jLabel2.setText("CavernCam Directory on FTP Site:");

        jLabel6.setText("Username:");

        jLabel7.setText("Password:");

        _rememberPasswordCheckBox.setSelected(true);
        _rememberPasswordCheckBox.setText("Remember Password");
        _rememberPasswordCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        _rememberPasswordCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

        _uploadImageDateCheckBox.setSelected(true);
        _uploadImageDateCheckBox.setText("Upload image date");
        _uploadImageDateCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        _uploadImageDateCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

        jLabel9.setText("Upload interval (seconds):");

        _uploadIntervalTextField.setEditable(false);
        _uploadIntervalTextField.setText("10");

        _uploadIntervalSlider.setMajorTickSpacing(1);
        _uploadIntervalSlider.setMaximum(10);
        _uploadIntervalSlider.setMinimum(1);
        _uploadIntervalSlider.setPaintLabels(true);
        _uploadIntervalSlider.setPaintTicks(true);
        _uploadIntervalSlider.setSnapToTicks(true);
        _uploadIntervalSlider.addChangeListener(new javax.swing.event.ChangeListener()
        {
            public void stateChanged(javax.swing.event.ChangeEvent evt)
            {
                _uploadIntervalSliderStateChanged(evt);
            }
        });

        _restartFtpSessionCheckBox.setText("Restart FTP Session between uploads");
        _restartFtpSessionCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        _restartFtpSessionCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel1)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel6)
                            .add(jLabel7)
                            .add(_uploadImageDateCheckBox)
                            .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel1Layout.createSequentialGroup()
                                    .add(jLabel9)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(_uploadIntervalTextField))
                                .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel2))
                            .add(_rememberPasswordCheckBox))
                        .add(28, 28, 28)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(_restartFtpSessionCheckBox)
                            .add(_uploadIntervalSlider, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 199, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, _passwordTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 199, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, _usernameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 199, Short.MAX_VALUE)
                            .add(_cavernCamDirTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 199, Short.MAX_VALUE)
                            .add(_ftpSiteTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 199, Short.MAX_VALUE))
                        .addContainerGap())))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(_ftpSiteTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(_cavernCamDirTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(9, 9, 9)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel6)
                    .add(_usernameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(14, 14, 14)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel7)
                    .add(_passwordTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(_rememberPasswordCheckBox)
                            .add(_restartFtpSessionCheckBox))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(_uploadImageDateCheckBox)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel9)
                            .add(_uploadIntervalTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(41, 41, 41)
                        .add(_uploadIntervalSlider, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Local Properties"));
        jLabel3.setText("Local Image Directory:");

        _localImageDirTextField.setEditable(false);

        _localImageDirBrowseButton.setText("...");
        _localImageDirBrowseButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                _localImageDirBrowseButtonActionPerformed(evt);
            }
        });

        jLabel4.setText("Image Type:");

        jLabel5.setText("Image Name Matching Substring:");

        _matchingSubstringTextField.setText("Uru");

        jLabel12.setText("Image Width:");

        _imageWidthTextField.setText("800");

        jLabel13.setText("Image Height:");

        _imageHeightTextField.setText("600");

        jLabel14.setText("\"Away\" Image:");

        _awayImageTextField.setEditable(false);

        _awayImageBrowseButton.setText("...");
        _awayImageBrowseButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                _awayImageBrowseButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel2Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jLabel4)
                        .add(41, 41, 41)
                        .add(_imageTypeComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanel2Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jLabel12)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(_imageWidthTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 54, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(10, 10, 10)
                        .add(jLabel13)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(_imageHeightTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 58, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel2Layout.createSequentialGroup()
                                .add(jLabel14)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(_awayImageTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 264, Short.MAX_VALUE))
                            .add(jPanel2Layout.createSequentialGroup()
                                .add(jLabel3)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(_localImageDirTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 228, Short.MAX_VALUE)))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(_awayImageBrowseButton)
                            .add(_localImageDirBrowseButton)))
                    .add(jPanel2Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jLabel5)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(_matchingSubstringTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 164, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(_localImageDirTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(_localImageDirBrowseButton)
                    .add(jLabel3))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(_imageTypeComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel4))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel5)
                    .add(_matchingSubstringTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel13)
                    .add(_imageHeightTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel12)
                    .add(_imageWidthTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(_awayImageTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(_awayImageBrowseButton)
                    .add(jLabel14)))
        );

        _okButton.setText("OK");
        _okButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                _okButtonActionPerformed(evt);
            }
        });

        _cancelButton.setText("Cancel");
        _cancelButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                _cancelButtonActionPerformed(evt);
            }
        });

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Screen Capture Properties"));
        _performScreenCaptureCheckBox.setText("Perform screen captures");
        _performScreenCaptureCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        _performScreenCaptureCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        _performScreenCaptureCheckBox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                _performScreenCaptureCheckBoxActionPerformed(evt);
            }
        });

        jLabel8.setText("Captured Image Quality:");

        _imageQualityTextField.setEditable(false);
        _imageQualityTextField.setText("95");

        _imageQualitySlider.setMajorTickSpacing(10);
        _imageQualitySlider.setMinimum(10);
        _imageQualitySlider.setMinorTickSpacing(5);
        _imageQualitySlider.setPaintLabels(true);
        _imageQualitySlider.setPaintTicks(true);
        _imageQualitySlider.setSnapToTicks(true);
        _imageQualitySlider.setValue(95);
        _imageQualitySlider.addChangeListener(new javax.swing.event.ChangeListener()
        {
            public void stateChanged(javax.swing.event.ChangeEvent evt)
            {
                _imageQualitySliderStateChanged(evt);
            }
        });

        _topYTextField.setText("0");

        jLabel10.setText("Top Y:");

        _leftXTextField.setText("0");

        jLabel11.setText("Left X:");

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jLabel8)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(_imageQualityTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 48, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(_performScreenCaptureCheckBox, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 185, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jLabel11)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(_leftXTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 37, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel10)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(_topYTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 38, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(_imageQualitySlider, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel10)
                    .add(jLabel11)
                    .add(_leftXTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(_topYTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(_performScreenCaptureCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(_imageQualitySlider, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(17, 17, 17)
                        .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(_imageQualityTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel8))))
                .addContainerGap())
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(_okButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 315, Short.MAX_VALUE)
                        .add(_cancelButton))
                    .add(jPanel3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(_okButton)
                    .add(_cancelButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void _awayImageBrowseButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event__awayImageBrowseButtonActionPerformed
    {//GEN-HEADEREND:event__awayImageBrowseButtonActionPerformed
        javax.swing.JFileChooser fc = new javax.swing.JFileChooser();
        fc.setDialogTitle("Select an Away image");
        fc.setFileFilter(new ImageFileFilter());
        int returnVal = fc.showOpenDialog(this);
        if (returnVal == javax.swing.JFileChooser.APPROVE_OPTION)
        {
            File file = fc.getSelectedFile();
            if (file.exists())
            {
                this._awayImageTextField.setText(file.getAbsolutePath());
            }
        }
        else
        {
            int result = javax.swing.JOptionPane.showConfirmDialog(this,"Do you want to clear the \"away\" image name?");
            if (result == javax.swing.JOptionPane.YES_OPTION)
            {
                this._awayImageTextField.setText("");
            }
        }
    }//GEN-LAST:event__awayImageBrowseButtonActionPerformed

    private void _uploadIntervalSliderStateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event__uploadIntervalSliderStateChanged
    {//GEN-HEADEREND:event__uploadIntervalSliderStateChanged
        this._uploadIntervalTextField.setText(String.valueOf(this._uploadIntervalSlider.getValue()));
    }//GEN-LAST:event__uploadIntervalSliderStateChanged

    private void _imageQualitySliderStateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event__imageQualitySliderStateChanged
    {//GEN-HEADEREND:event__imageQualitySliderStateChanged
        this._imageQualityTextField.setText(String.valueOf(this._imageQualitySlider.getValue()));
    }//GEN-LAST:event__imageQualitySliderStateChanged

    private void _performScreenCaptureCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event__performScreenCaptureCheckBoxActionPerformed
    {//GEN-HEADEREND:event__performScreenCaptureCheckBoxActionPerformed
        boolean bEnable = this._performScreenCaptureCheckBox.isSelected();
        this._imageQualitySlider.setEnabled(bEnable);
        this._leftXTextField.setEnabled(bEnable);
        this._topYTextField.setEnabled(bEnable);
    }//GEN-LAST:event__performScreenCaptureCheckBoxActionPerformed

    private void _cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event__cancelButtonActionPerformed
    {//GEN-HEADEREND:event__cancelButtonActionPerformed
        this.dispose();
    }//GEN-LAST:event__cancelButtonActionPerformed

    private void _okButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event__okButtonActionPerformed
    {//GEN-HEADEREND:event__okButtonActionPerformed

        // Check that all the fields are set
        try
        {
            readFromFields();
        }
        catch(Exception ex)
        {
            javax.swing.JOptionPane.showMessageDialog(this,ex.getMessage(),"Missing Information",javax.swing.JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        // Save the properties file
        CCUProperties.instance().write();
        this.dispose();
    }//GEN-LAST:event__okButtonActionPerformed

    private void _localImageDirBrowseButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event__localImageDirBrowseButtonActionPerformed
    {//GEN-HEADEREND:event__localImageDirBrowseButtonActionPerformed

        javax.swing.JFileChooser fc = new javax.swing.JFileChooser();
        fc.setDialogTitle("Select a local image directory");
        fc.setFileSelectionMode(javax.swing.JFileChooser.DIRECTORIES_ONLY);
        int returnVal = fc.showOpenDialog(this);
        if (returnVal == javax.swing.JFileChooser.APPROVE_OPTION)
        {
            File file = fc.getSelectedFile();
            if (file.isDirectory())
            {
                this._localImageDirTextField.setText(file.getAbsolutePath());
            }
        }
    }//GEN-LAST:event__localImageDirBrowseButtonActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton _awayImageBrowseButton;
    private javax.swing.JTextField _awayImageTextField;
    private javax.swing.JButton _cancelButton;
    private javax.swing.JTextField _cavernCamDirTextField;
    private javax.swing.JTextField _ftpSiteTextField;
    private javax.swing.JTextField _imageHeightTextField;
    private javax.swing.JSlider _imageQualitySlider;
    private javax.swing.JTextField _imageQualityTextField;
    private javax.swing.JComboBox _imageTypeComboBox;
    private javax.swing.JTextField _imageWidthTextField;
    private javax.swing.JTextField _leftXTextField;
    private javax.swing.JButton _localImageDirBrowseButton;
    private javax.swing.JTextField _localImageDirTextField;
    private javax.swing.JTextField _matchingSubstringTextField;
    private javax.swing.JButton _okButton;
    private javax.swing.JTextField _passwordTextField;
    private javax.swing.JCheckBox _performScreenCaptureCheckBox;
    private javax.swing.JCheckBox _rememberPasswordCheckBox;
    private javax.swing.JCheckBox _restartFtpSessionCheckBox;
    private javax.swing.JTextField _topYTextField;
    private javax.swing.JCheckBox _uploadImageDateCheckBox;
    private javax.swing.JSlider _uploadIntervalSlider;
    private javax.swing.JTextField _uploadIntervalTextField;
    private javax.swing.JTextField _usernameTextField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    // End of variables declaration//GEN-END:variables
    
    private class ImageFileFilter extends javax.swing.filechooser.FileFilter
    {
        public String getDescription()
        {
            return CCUProperties.instance().getImageType() + " Files";
        }
        public boolean accept(File whichFile)
        {
            if (whichFile.isDirectory())
            {
                return true;
            }
            
            String name = whichFile.getName().trim().toLowerCase();
            return (name.endsWith(CCUProperties.instance().getImageType()));
        }
    }
    
}
